"use strict";
exports.__esModule = true;
var tslib_1 = require("tslib");
var utils = require("eslint-plugin-prettier");
var prettier = require("prettier");
var tslint = require("tslint");
// tslint:disable:max-classes-per-file no-use-before-declare restrict-plus-operands
var Rule = (function (_super) {
    tslib_1.__extends(Rule, _super);
    function Rule() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Rule.prototype.apply = function (source_file) {
        var _a = this.ruleArguments[0], raw_options = _a === void 0 ? {} : _a;
        var options = tslib_1.__assign({ parser: 'typescript' }, raw_options);
        return this.applyWithWalker(new Walker(source_file, this.ruleName, options));
    };
    return Rule;
}(tslint.Rules.AbstractRule));
exports.Rule = Rule;
var Walker = (function (_super) {
    tslib_1.__extends(Walker, _super);
    function Walker() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Walker.prototype.walk = function (source_file) {
        var _this = this;
        var source = source_file.getFullText();
        var formatted = prettier.format(source, this.options);
        if (source === formatted) {
            return;
        }
        utils.generateDifferences(source, formatted).forEach(function (difference) {
            var operation = difference.operation, start = difference.offset, _a = difference.deleteText, delete_text = _a === void 0 ? '' : _a, _b = difference.insertText, insert_text = _b === void 0 ? '' : _b;
            var end = start + delete_text.length;
            var delete_code = utils.showInvisibles(delete_text);
            var insert_code = utils.showInvisibles(insert_text);
            switch (operation) {
                case "insert" /* Insert */:
                    _this.addFailureAt(start, 1, "Insert `" + insert_code + "`", tslint.Replacement.appendText(start, insert_text));
                    break;
                case "delete" /* Delete */:
                    _this.addFailure(start, end, "Delete `" + delete_code + "`", tslint.Replacement.deleteFromTo(start, end));
                    break;
                case "replace" /* Replace */:
                    _this.addFailure(start, end, "Replace `" + delete_code + "` with `" + insert_code + "`", tslint.Replacement.replaceFromTo(start, end, insert_text));
                    break;
                // istanbul ignore next
                default:
                    throw new Error("Unexpected operation '" + operation + "'");
            }
        });
    };
    return Walker;
}(tslint.AbstractWalker));
